///////////////////////////////////////////////////////////////////////////////
// DbgPrnHk.h

#ifndef __DEBUG_PRINT_HOOK_H__
#define __DEBUG_PRINT_HOOK_H__

extern "C" {

#pragma pack(push, 1)

#ifndef USER_MODE

#include <ntddk.h>                  // various NT definitions
#include <stddef.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include "ntddk_ex.h"
#include "tools.h"

#ifdef POOL_TAGGING
#ifdef ExAllocatePool
#undef ExAllocatePool
#endif //ExAllocatePool
#define ExAllocatePool(a,b) ExAllocatePoolWithTag(a,b,'kHPD')
#endif //POOL_TAGGING

#define NT_DBGPRNHK_REG_PATH        L"\\REGISTRY\\MACHINE\\SYSTEM\\CurrentControlSet\\Services\\DbgPrnHk"

#else //USER_MODE

#define NT_DBGPRNHK_REG_PATH        L"SYSTEM\\CurrentControlSet\\Services\\DbgPrnHk"

#endif //USER_MODE

#define NT_DBGPRNHK_REG_TAG_VAL              L"Tag"
#define NT_DBGPRNHK_DRV_TAG                  0x0000DBC0
#define NT_DBGPRNHK_REG_GROUP_VAL            L"Group"

// Device names
#define NT_DbgPrnHk_DOS_NAME     ("\\DosDevices\\DbgPrnHk")
#define NT_DbgPrnHk_NAME         ("\\Device\\DbgPrnHk")
#define NT_DbgPrnHk_NAME_W       (L"\\Device\\DbgPrnHk")
#define NT_DbgPrnHk_USER_NAME    ("\\\\.\\DbgPrnHk")
#define NT_DbgPrnHk_USER_NAME_W  (L"\\\\.\\DbgPrnHk")
#define NT_DbgPrnHk_SVC_NAME     ("DbgPrnHk")

#ifndef CTL_CODE
#include "winioctl.h"
#endif //CTL_CODE

#include "version.h"

#define DPHK_CTL_CODE_X(a,b)    CTL_CODE(FILE_DEVICE_UNKNOWN, a,b, FILE_ANY_ACCESS )

#define IOCTL_DbgPrnHk_GetMessages           DPHK_CTL_CODE_X(0x801, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_SetMessageBufferSize  DPHK_CTL_CODE_X(0x802, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_GetVersion            DPHK_CTL_CODE_X(0x803, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_PostMessage           DPHK_CTL_CODE_X(0x804, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_GetRdtscCalibration   DPHK_CTL_CODE_X(0x805, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_GetRdtsc              DPHK_CTL_CODE_X(0x806, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_CanUnload             DPHK_CTL_CODE_X(0x807, METHOD_BUFFERED)
#define IOCTL_DbgPrnHk_PostMessageEx         DPHK_CTL_CODE_X(0x808, METHOD_BUFFERED)

#define MAX_MSG_SIZE  (128-sizeof(LARGE_INTEGER)-sizeof(PVOID)*2-sizeof(USHORT)-sizeof(UCHAR)*2-sizeof(UCHAR)*4)

#define TimeStampType_SysPerfCounter   0x00
#define TimeStampType_RdtscPerfCounter 0x01
#define TimeStampType_SysTime          0x02
#define TimeStampType_Max              0x02

typedef struct _DbgPrnHk_GetMessages_USER_OUT {
    LARGE_INTEGER TimeStamp;
    PVOID         ProcessId;
    PVOID         ThreadId;
    USHORT        Length;
    UCHAR         Irql;
    UCHAR         CpuNumber;
    UCHAR         TimeStampType:2;
    UCHAR         CallerMode:1;
    UCHAR         Reserved0:5;
    UCHAR         Reserved[3];
    CHAR          Msg[MAX_MSG_SIZE];
} DbgPrnHk_GetMessages_USER_OUT, *PDbgPrnHk_GetMessages_USER_OUT;

typedef struct _DbgPrnHk_SetMessageBuffer_USER_IN {
    ULONG BufferSize;
} DbgPrnHk_SetMessageBuffer_USER_IN, *PDbgPrnHk_SetMessageBuffer_USER_IN;

typedef struct _DbgPrnHk_GetVersion_USER_OUT {
    ULONG         Major;
    ULONG         Minor;
    ULONG         Sub;
} DbgPrnHk_GetVersion_USER_OUT, *PDbgPrnHk_GetVersion_USER_OUT;

typedef struct _DbgPrnHk_GetRdtscCalibration_USER_IN {
    BOOLEAN       Recalibrate;
    BOOLEAN       ChangeType;
    UCHAR         TimeStampType;  // valid if ChangeType == TRUE
} DbgPrnHk_GetRdtscCalibration_USER_IN, *PDbgPrnHk_GetRdtscCalibration_USER_IN;

typedef struct _DbgPrnHk_GetRdtscCalibration_USER_OUT {
    __int64       RdtscTimeStampCalibration0;
    __int64       SysTimeStampCalibration0;
    __int64       RdtscTimeStampCalibration;
    __int64       SysTimeStampCalibration;
    UCHAR         TimeStampType;
} DbgPrnHk_GetRdtscCalibration_USER_OUT, *PDbgPrnHk_GetRdtscCalibration_USER_OUT;

typedef DbgPrnHk_GetMessages_USER_OUT  DbgPrnHk_PostMessageEx_USER_IN;
typedef PDbgPrnHk_GetMessages_USER_OUT PDbgPrnHk_PostMessageEx_USER_IN;

#ifndef USER_MODE

// Function declarations
extern NTSTATUS DriverEntry(IN PDRIVER_OBJECT DriverObject, IN PUNICODE_STRING RegistryPath);
extern NTSTATUS DbgPrnHkCreateClose(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp);
extern NTSTATUS DbgPrnHkDeviceControl(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp);
extern VOID     DbgPrnHkUnload(IN PDRIVER_OBJECT DriverObject);

#endif //USER_MODE

/* driver registry options */

#define NT_DBGPRNHK_REG_CHECK_IRQL_VAL       L"CheckIrql"
#define NT_DBGPRNHK_REG_BUFFER_SIZE_VAL      L"BufferSize"
#define NT_DBGPRNHK_REG_BLOCK_MSG_VAL        L"DoNotPassMessagesDown"
#define NT_DBGPRNHK_REG_STOP_OVERFLOW_VAL    L"StopOnBufferOverflow"
#define NT_DBGPRNHK_REG_TSTAMP_TYPE_VAL      L"TimeStampType"

/*
typedef struct _DBGPRINT_HOOK_OPTIONS {
    PWCHAR OptionName;
    ULONG  OptionType;
    ULONG  UpdateFlag;
    union {
        ULONG OptDword;
        PWCHAR OptStr;
    } DefValue;
    union {
        ULONG OptDword;
        PWCHAR OptStr;
    } Value;
} DBGPRINT_HOOK_OPTIONS, *PDBGPRINT_HOOK_OPTIONS;
*/
//extern DBGPRINT_HOOK_OPTIONS DbgPrintHook_Options[];

typedef struct _DBGPRNHK_INTERNAL_STATE {

    DbgPrnHk_GetVersion_USER_OUT   Ver;

    PDbgPrnHk_GetMessages_USER_OUT MsgBuffer;
    LARGE_INTEGER ReadPosition;
    LARGE_INTEGER WritePosition;
    ULONG   BufferSize;
    ULONG   BufferSizeMask;
    ULONG   QueueSize;
    ULONG   MaxQueueSize;
    BOOLEAN CheckIrql;
    BOOLEAN NoPassMsgDown;
    BOOLEAN StopOnBufferOverflow;
    BOOLEAN LoggingPaused;
    UCHAR TimeStampType;

} DBGPRNHK_INTERNAL_STATE, *PDBGPRNHK_INTERNAL_STATE;

#pragma pack(pop)

};

#endif // __DEBUG_PRINT_HOOK_H__

